#!/bin/bash
# Author:       Miguel Gea Milvaques <gea_mig@gva.es>
# Description:  Install gettext data.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#


# library
[ "$LLXCFG_DEV_LIB" ] || LLXCFG_DEV_LIB="/usr/share/lliurex/llxcfg-dev/llxcfg-dev.sh"
[ -f "${LLXCFG_DEV_LIB}" ] || exit 1
. ${LLXCFG_DEV_LIB}


GETTEXT_MD5_DIR="debian/"

dl_getgettext_dir(){
	package="$1"
	GETTEXT_DIR=""
	# Obtain the directory containing the po files
	[ -e "debian/gettextdir" ] && GETTEXT_DIR="$(cat debian/gettextdir)"
	[ -e "debian/${package}.gettextdir" ] && GETTEXT_DIR="$(cat debian/${package}.gettextdir)"
	[ -n "$GETTEXT_DIR" ] || GETTEXT_DIR="lliurex-po/$package"
	echo "$GETTEXT_DIR"
}
dl_installgettext(){
	package="$1"
	GETTEXT_DIR="$(dl_getgettext_dir "$package")"
	# Obtain the gettext domain
	if [ -d "$GETTEXT_DIR" ]; then
		FILE="$(find $GETTEXT_DIR -name "*.pot")"
		[ $(echo $FILE|wc -w) -eq 1 ] || dl_exit_error "Incorrect .pot files count in $GETTEXT_DIR: "$(echo $FILE|wc -w)" pot file(s)"
		GETTEXT_DOMAIN="$(basename $(find $GETTEXT_DIR -name "*.pot") .pot)"
		for i in $GETTEXT_DIR/*.po ; do
			LNG="$(basename $i .po)"
			mkdir -p debian/${package}/usr/share/locale/${LNG}/LC_MESSAGES/
			msgfmt "$i" -o "debian/${package}/usr/share/locale/${LNG}/LC_MESSAGES/${GETTEXT_DOMAIN}.mo"
		done
	else
		echo "********"
		echo "Warning: There are no $GETTEXT_DIR directory. Avoid."
		echo "********"
	fi

}


# Function:    dl_getpot, based in cddt-update-pot
# Description: Script to update .pot and .po files of cddt packages.
# Author:      Sergio Talens-Oliag <sto@debian.org>
# Copyright:   (c) 2006 Sergio Talens-Oliag <sto@debian.org>
# SVN Id:      $Id: cddt-update-pot 313 2006-02-04 12:39:13Z sto $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA

dl_getpot(){
	# Commands
	XGETTEXT="xgettext"

	# Initial tests
	if [ ! -f "./debian/rules" -o ! -d "./lliurex-po" ]; then
	  echo "Call this script from a package toplevel dir" >&2
	  exit 1
	fi
	if [ "$#" -ne "3" ]; then
	  echo "Usage: $0 textdomain basedir outputdir" >&2
	  exit 1
	fi
	textdomain="$1"
	basedir="$2"
	outputdir="$3"

	if [ ! -d "$basedir" ]; then
	  echo "Invalid basedir '$basedir'" >&2
	  exit 1
	fi
	# Find files
	SHELL_FILES=$(
	  find "${basedir}" -path "*/.svn*" -prune -o -type f -exec file {} \; \
	  | sed -n -e '/shell script/ { s/^\(.*\): .*shell script.*$/\1/; P } '
	)
	PYTHON_FILES=$( 
	  find "${basedir}" -path "*/.svn*" -prune -o -type f -exec file {} \; \
	  | sed -n -e '/^.*\.py:/ { s/^\(.*\): .*$/\1/; P };
	    /python script/ { s/^\(.*\): .*python script.*$/\1/; P } '
	)
	PHP_FILES=$(
	  find "${basedir}" -path "*/.svn*" -prune -o -type f -exec file {} \; \
	  | sed -n -e '/^.*\.php:/ { s/^\(.*\): .*$/\1/; P };
	    /PHP script/ { s/^\(.*\): .*PHP script.*$/\1/; P } '
	)
	GLADE_FILES=$(
	  find "${basedir}" -path "*/.svn*" -prune -o -name "*.glade" -type f -exec file {} \; \
	  | sed -n -e '/XML/ { s/^\(.*\): .*XML.*$/\1/; P } '
	)
	# Remove old pot to avoid keeping old strings
	rm -f "${outputdir}/${textdomain}.pot"
	# Process shell files
cat << EOF > "${outputdir}/${textdomain}.pot"
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\n"
"Report-Msgid-Bugs-To: \n"
"POT-Creation-Date: 2008-11-27 12:47+0100\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: LANGUAGE <LL@li.org>\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=CHARSET\n"
"Content-Transfer-Encoding: 8bit\n"
EOF
	if [ -n "$SHELL_FILES" ]; then
	  # Generate POT file
	  $XGETTEXT -L Shell -o "${outputdir}/${textdomain}.pot" $SHELL_FILES
	  
	  # Compare source and pot file gettext invocations
	  
	  # This check is done because I've detected that sometimes the xgettext
	  # parser misses some gettext strings on scripts; as a general rule what I do
	  # now is to call 'gettext' using backquotes (`) inside double quotes when
	  # assigning its value to a variable, like this:
	  #
	  #    VARIABLE="` gettext "TEXT" `"
	  #
	  # and I call 'xgettext' using $() and without double quotes:
	  #
	  #    VARIABLE=$( xgettext "TEXT \$varname" )
	  #
	  # Following those rules is easy and until now everything seems to work OK.
	  #
	  # Of course, the test is still here to detect xgettext failures if the
	  # previous rules are not followed
	  
	  # Create temporary files
	  STMPF="`tempfile -p src_ -s .updt_pot`"
	  PTMPF="`tempfile -p pot_ -s .updt_pot`"
	  
	  # Try to detect the gettext calls on shell scripts usign sed
	  grep -H -n "^[[:blank:]]*[^#]*gettext[[:blank:]]" $SHELL_FILES \
	  | cut -d':' -f1,2 | sort -u >> "$STMPF"
	  
	  # Get POT file entries
	  sed -n -e "/^#:/ { s/^#: //; P; }" "${outputdir}/${textdomain}.pot" 2> /dev/null \
	    | awk '{ for (n=1;n<=NF;n++) { print $n } }' \
	    | sort -u > "$PTMPF"
	  
	  # Compute number of strings
	  src_strings="`cat "$STMPF" | wc -l`"
	  pot_strings="`cat "$PTMPF" | wc -l`"
	  
	  # Print some data if the gettext strings differ
	  if [ "$src_strings" -ne "$pot_strings" ]; then
	    printf "%-44s %d\n" "strings on Shell files:" "$src_strings"
	    printf "%-44s %d\n" "strings on '${textdomain}.pot':" "$pot_strings"
	    echo "string differences:"
	    diff -U 0 "$STMPF" "$PTMPF";
	  fi

	  # Remove TMP files
	  rm -f "$STMPF" "$PTMPF"
	fi


	# Process python files
	if [ -n "$PYTHON_FILES" ]; then
	  [ -f "${outputdir}/${textdomain}.pot" ] && merge="-j" || merge=""
	  $XGETTEXT -L Python $merge -o "${outputdir}/${textdomain}.pot" $PYTHON_FILES
	fi

	# Process PHP files
	if [ -n "$PHP_FILES" ]; then
	  [ -f "${outputdir}/${textdomain}.pot" ] && merge="-j" || merge=""
	  $XGETTEXT --from-code=UTF-8 --keyword=N_ -L PHP $merge \
	    -o "${outputdir}/${textdomain}.pot" $PHP_FILES
	fi
	
	# Process Glade files
	if [ -n "$GLADE_FILES" ]; then
	  [ -f "${outputdir}/${textdomain}.pot" ] && merge="-j" || merge=""
	  $XGETTEXT -L Glade $merge -o "${outputdir}/${textdomain}.pot" $GLADE_FILES
	fi

	# Update .po files
	#for f in `ls ${outputdir}/*.po 2> /dev/null`; do
	#  echo "Updating '$f' file"
	#  msgmerge  -U $f ${outputdir}/${textdomain}.pot
	#  stats="`msgfmt -o /dev/null --statistics $f 2>&1`"
	#  echo "$f: $stats"
	#done
}

dl_potMD5SUM(){
	DOMAIN="$(basename $1 .pot)"
	TMPDIR="$(mktemp -d)"
	msgattrib -s $1 -o $TMPDIR/$DOMAIN.po 
       	po2txt --progress=none -i "$TMPDIR/${DOMAIN}.po"|md5sum|awk '{print $1}'
	rm -Rf $TMPDIR
}

dl_compareMD5pot(){
	TMPDIR="$(mktemp -d)"
	touch "$GETTEXT_MD5_DIR/lliurex-po.gettextinfo"
	dl_getpot "all" "./" "$TMPDIR"
	NEW_MD5="$(dl_potMD5SUM "$TMPDIR/all.pot")"
	OLD_MD5="$(cat "$GETTEXT_MD5_DIR/lliurex-po.gettextinfo")"
	if [ "$NEW_MD5" != "$OLD_MD5" ]; then
		touch "debian/PLEASE.UPDATE.POT"
	fi
	rm -Rf $TMPDIR
}

