# Author:	Luis Garcia Gisbert <garcia_luigis@gva.es>
# Description:	common llxcfg-dev functions
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#


# library
LLX_COMMON="/usr/share/lliurex/llxcfg-common/llxcfg-common.sh"
[ -z "$LLXCFG_COMMON_LIB" ] || LLX_COMMON="$LLXCFG_COMMON_LIB"
[ -f "${LLX_COMMON}" ] || exit 1
. ${LLX_COMMON}

# Public vars
declare -a DL_PKG_LINE
CDD_FULL_PREFIX="lliurex-cdd"
CDD_PATH="/usr/share/lliurex-cdd"
LLX_PATH="/usr/share/llxcfg-info"

# Functions
dl_exit_error(){
   echo "$@" >&2
   exit 1
}

dl_get_version(){
   [ -f "debian/changelog" ] && head -1 debian/changelog |sed -ne "s%^[^(]*(\([^)]*\)).*%\1%p"
}

dl_deb_test(){
   [ -d "debian" -a -f "debian/control" ]  && return 0
   dl_exit_error "oops! Are you really building a Debian package?"
}

dl_control_field(){
    grep -q "^$2:" "$1" || echo "$2: $3" >> "$1"
}

dl_list_binpkgs(){
   REG_EXP="^Package:"
   [ -f "debian/control" ] && sed -ne "/${REG_EXP}/s%${REG_EXP}[[:space:]]*%%p" "debian/control"
}

dl_first_pkg(){
   dl_list_binpkgs |head -1
}

dl_show_pkg(){
   [ -r "debian/control" ] || return 1
   PKG_NAME="$1"
   [ "$PKG_NAME" ] || PKG_NAME="$(dl_first_pkg)"
   if [ "$PKG_NAME" = "SOURCE" ] ; then
      REGEXP="^Source:"
      REGEXP2="^Package:"
   else
      REGEXP="^Package:[[:blank:]]*${PKG_NAME}[[:blank:]]*$"
      REGEXP2="$REGEXP"
   fi
   NOTDEL_CMD='!d'
   sed -ne "/$REGEXP/,/^Package:/{/^Package:/{/${REGEXP2}/$NOTDEL_CMD};/^[[:blank:]]*$/d;p}" debian/control
}

dl_add_mant_script(){
   MANT_SCR="$1"
   shift
   COMMENT="$1"
   shift
   PKG_NAME="$1"
   shift
   SCR_FILE="debian/$PKG_NAME.$MANT_SCR.debhelper"
   echo "## automatically generated by $COMMENT (llxcfg-dev)" >> "$SCR_FILE"
   while [ "$1" ] ; do
      echo "$1" >> "$SCR_FILE"
      shift
   done
}

dl_add_postinst(){
   dl_add_mant_script postinst "$@"
}

dl_add_prerm(){
   dl_add_mant_script prerm "$@"
}

dl_add_postrm(){
   dl_add_mant_script postrm "$@"
}

dl_show_field(){
   FLD_NAME="$1"
   # check X-field ...
   PKG_NAME="$2"
   [ "$FLD_NAME" ] || return 1
   REG_FLD="^${FLD_NAME}:[[:blank:]]*"
   if echo "$FLD_NAME" |grep -q "^X-" ; then
      REG_FLD="^X[BSC]\{0,3\}-${FLD_NAME#X-}:[[:blank:]]*"
   fi
   dl_show_pkg "$PKG_NAME" |sed -ne "/$REG_FLD/s%$REG_FLD%%p"
}

dl_read_pkgline(){
   DL_PKG_LINE=("$1" "$2" "$3")
}

dl_install_scripts(){
   p="$1"
   CURRENT_TYPES=$(find debian -maxdepth 1 -xtype f -name "?*-scripts" |sed -e "s%^debian/%%;s%^[^.]\+\.%%;s%-scripts$%%" |sort -u)
   for t in ${CURRENT_TYPES} ; do
      INST_LIST="debian/${p}.${t}-scripts"
      if [ "$p" = "$(dl_first_pkg)" ] ; then
         ALT_INST_LIST="debian/${t}-scripts"
         [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
      fi
      if [ -s "$INST_LIST" ] ; then
         cat "$INST_LIST" |while read line ; do
            dl_read_pkgline $line
            SCR_FILE="${DL_PKG_LINE[0]}"
            SCR_NAME="$(basename ${SCR_FILE})"
            SCR_NUM="${DL_PKG_LINE[1]}"
            [ ! -e "$SCR_FILE" ] && dl_exit_error "${t} script \"${SCR_FILE}\" not found"
            mkdir -p "debian/$p/$(llxscr_dir "$t")"
            mkdir -p "debian/$p/$(llxscr_endir "$t")"
            cp "$SCR_FILE" "debian/$p/$(llxscr_file "$t" "$SCR_NAME")"
            chmod +x "debian/$p/$(llxscr_file "$t" "$SCR_NAME")"
            ln -s "$(llxscr_relfile "${t}" "$SCR_NAME")" "debian/$p/$(llxscr_enfile "${t}" "$SCR_NAME" $SCR_NUM)"
            dl_add_depends_runtime "$p"
         done
      fi
   done
}


dl_install_cpkgs(){
   p="$1"
   INST_LIST="debian/${p}.cpkgs"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/cpkgs"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      cat "$INST_LIST" |while read line ; do
         dl_read_pkgline $line
         CP_DIR="${DL_PKG_LINE[0]}"
         CP_NUM="${DL_PKG_LINE[1]}"
         CP_NAM="${DL_PKG_LINE[2]}"
         [ -z "${CP_NAM}" ] && CP_NAM="$(basename ${CP_DIR})"
         [ ! -d "$CP_DIR" ] && dl_exit_error "cpkg directory \"${CP_DIR}\" not found"
         for d in ${CPKG_DIRS} ; do
            mkdir -p "debian/$p/${d}/${CP_NAM}"
         done
         mkdir -p "debian/$p/${CPKG_ENABLED}"
         mkdir -p "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}"
         rsync -ax "${CP_DIR}/" "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}" || dl_exit_error "Error installing files from cpkg directory \"${CP_DIR}\""
         for s in ${CPKG_SCRNAMES} ; do
            [ -f "debian/${p}/${CPKG_ORIGINAL}/${CP_NAM}/${s}" ] && chmod +x "debian/${p}/${CPKG_ORIGINAL}/${CP_NAM}/${s}"
         done
         # add current binary to packages.list (if any)
         [ ! -r "$CP_DIR/$CPKG_PKGS_FILE" ] || { cat "$CP_DIR/$CPKG_PKGS_FILE" ; echo ""; echo "$p"; } |sed -e "/^[[:blank:]]*$/d" |sort -u > "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/$CPKG_PKGS_FILE"
         # add cpkg to update-list
         dl_add_update "$p" "${CP_NAM}"

         ln -s "$(cpkg_reldir "${CP_NAM}")" "debian/$p/$(cpkg_endir "$CP_NAM" $CP_NUM)"
         CPKG_CONTROL="debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${CPKG_CONTROL_FILE}"
         [ -f "${CPKG_CONTROL}" ] || touch "${CPKG_CONTROL}"
         dl_control_field "${CPKG_CONTROL}" "Version" "$(dl_get_version)"
         dl_control_field "${CPKG_CONTROL}" "Upgrade-actions" "deconfigure"
         [ "$CP_NUM" ] && dl_control_field "${CPKG_CONTROL}" "Default-order" "$CP_NUM"
         CPKG_CONFFILES="debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${CPKG_CONF_FILE}"
         if [ ! -f "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${CPKG_INSTALL_FILE}" ] ; then
            if [ -d "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${DEFAULT_SOURCE}/" ] ; then
               find "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${DEFAULT_SOURCE}/" -type f -mindepth 1 -printf "/%P\n" >> "$CPKG_CONFFILES"
            fi
         fi
         if [ ! -f "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${CPKG_TEMPLATES_FILE}" ] ; then 
            if [ -d "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${DEFAULT_TEMPLATE}/" ] ; then
               find "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/${DEFAULT_TEMPLATE}/" -type f -mindepth 1 -printf "/%P\n" >> "$CPKG_CONFFILES"
            fi
         fi
         if [ -s "$CPKG_CONFFILES" ] ; then
            TMP_FILE="$(tempfile)"
            sort -u "$CPKG_CONFFILES" > "$TMP_FILE"
            cat "$TMP_FILE" > "$CPKG_CONFFILES"
            rm -f "$TMP_FILE"
         fi
         dl_add_depends_runtime "$p"
         # copy to run dir (/info) ?
#         rsync -ax "debian/$p/${CPKG_ORIGINAL}/${CP_NAM}/" "debian/$p/$(cpkg_dir "${CP_NAM}")" || dl_exit_error "Error installing files from cpkg directory \"${CP_DIR}\""
#         CPKG_LIST="debian/$p/$(cpkg_dir "${CP_NAM}")/${CPKG_LIST_FILE}"
#         touch "$CPKG_LIST" 
#         find "debian/$p/$(cpkg_dir "${CP_NAM}")" -mindepth 1 -printf "%P\n" >> "$CPKG_LIST"
      done
   fi
}

dl_install_vpkgs(){
   p="$1"
   INST_LIST="debian/${p}.vpkgs"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/vpkgs"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      cat "$INST_LIST" |while read line ; do
         dl_read_pkgline $line
         CP_DIR="${DL_PKG_LINE[0]}"
         CP_NUM="${DL_PKG_LINE[1]}"
         CP_NAM="${DL_PKG_LINE[2]}"
         [ -z "${CP_NAM}" ] && CP_NAM="$(basename ${CP_DIR})"
         [ ! -d "$CP_DIR" ] && dl_exit_error "vpkg directory \"${CP_DIR}\" not found"
         mkdir -p "debian/$p/${VPKG_INFO}"
         mkdir -p "debian/$p/${VPKG_ENABLED}"
         rsync -ax "${CP_DIR}/" "debian/$p/$(vpkg_dir "${CP_NAM}")" || dl_exit_error "Error installing files from vpkg directory \"${CP_DIR}\""
         for s in ${VPKG_SCRNAMES} ; do
            [ -f "debian/${p}/$(vpkg_dir "${CP_NAM}/${s}")" ] && chmod +x "debian/${p}/$(vpkg_dir "${CP_NAM}/${s}")"
         done
         ln -s "$(vpkg_reldir "${CP_NAM}")" "debian/$p/$(vpkg_endir "$CP_NAM" $CP_NUM)"
      done
   fi
}

dl_install_vars(){
   # creates a 'default' vars file named "<binary_package_name>-vars that contains only static (default) vars definition
   p="$1"
   INST_LIST="debian/${p}.vars"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/vars"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      CP_NAM="${p}-vars"
      mkdir -p "debian/$p/${LLXVARS_DIR1}"
      cp "$INST_LIST" "debian/$p/${LLXVARS_DIR1}/${CP_NAM}"
      dl_add_depends_runtime "$p"
   fi
   return 0
}

dl_install_config(){
   # installs static llxcfg-config files and creates an empty directory tree
   # in 'admin' and 'progs' llxcfg-config dirs
   p="$1"
   INST_LIST="debian/${p}.confdirs"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/confdirs"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      mkdir -p "debian/$p/$LCFG_STATIC_DIR/"
      cat "$INST_LIST" |while read CDIR ; do
         if [ "$CDIR" ] ; then
            [ ! -d "$CDIR" ] && dl_exit_error "config directory \"${CDIR}\" not found"
            rsync -ax "$CDIR/" "debian/$p/$LCFG_STATIC_DIR/"
            find "$CDIR/" -type d -mindepth 1 -printf "/%P\n" |while read s ; do
            dl_add_depends_runtime "$p"
               for d in $LCFG_ADMIN_DIR $LCFG_PROGS_DIR ; do
                  mkdir -p "debian/$p/$d/$s"
               done
            done
         fi
      done
   fi
   return 0
}

dl_install_skel(){
   # installs 'jumble' skel files
   p="$1"
   INST_LIST="debian/${p}.skel"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/skel"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      mkdir -p "debian/$p/${LLXCFG_JUMBLE_SKEL}/"
      cat "$INST_LIST" |while read SDIR ; do
         if [ "$SDIR" ] ; then
            [ ! -d "$SDIR" ] && dl_exit_error "jumble skel directory \"${SDIR}\" not found"
            rsync -ax "$SDIR/" "debian/$p/${LLXCFG_JUMBLE_SKEL}/"
            dl_add_depends_runtime "$p"
            dl_add_update "$p" "jumble"
         fi
      done
   fi
   return 0
}

dl_install_desktops(){
   # installs desktops-keeper data
   p="$1"
   INST_LIST="debian/${p}.desktopdirs"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/desktopdirs"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      mkdir -p "debian/$p/${LLXCFG_DESKTOPS_DIR}/$p"
      cat "$INST_LIST" |while read line ; do
         dl_read_pkgline $line
         SDIR="${DL_PKG_LINE[0]}"
         PRIO="${DL_PKG_LINE[1]}"

         if [ "$SDIR" ] ; then
            [ ! -d "$SDIR" ] && dl_exit_error "desktops-keeper directory \"${SDIR}\" not found"
            rsync -ax "$SDIR/" "debian/$p/${LLXCFG_DESKTOPS_DIR}/$p"
            dl_add_postinst "dllxh_installdesktops" "$p" "$LLXCFG_DESKTOPS_ADD ${LLXCFG_DESKTOPS_DIR}/$p $PRIO"
            dl_add_prerm "dllxh_installdesktops" "$p" "$LLXCFG_DESKTOPS_DEL ${LLXCFG_DESKTOPS_DIR}/$p"
            dl_add_depends "$p" "llxcfg-runtime" "llxcfg-desktops-keeper"
         fi
      done
   fi
   return 0
}

dl_sort_clean(){
   SFILE="$1"
   [ "$SFILE" ] || SFILE="-"
   sed -e "/^#/d;/^[[:blank:]]*$/d" "$SFILE" |sort -u
}

dl_add_update(){
   p="$1"
   shift
   mkdir -p "debian/$p/${CPKG_UPDATE_DIR}"
   CUPDATE_TMP="$(tempfile)"
   while [ "$1" ] ; do
         echo "$1" >> "$CUPDATE_TMP"
         shift
   done
   if [ -s "$CUPDATE_TMP" ] ; then
      [ -s "debian/$p/${CPKG_UPDATE_DIR}/$p" ] || cat "debian/$p/${CPKG_UPDATE_DIR}/$p" >> "$CUPDATE_TMP"
      dl_sort_clean "$CUPDATE_TMP" > "debian/$p/${CPKG_UPDATE_DIR}/$p"
   fi
    
}

dl_cpkg_update(){
   # installs desktops-keeper data
   p="$1"
   INST_LIST="debian/${p}.cpkg-update"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/cpkg-update"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      dl_add_update "$p" $(dl_sort_clean "$INST_LIST")
   fi
   return 0
}

dl_install_templates(){
   # installs 'jumble' template files
   p="$1"
   INST_LIST="debian/${p}.tmpl"
   if [ "$p" = "$(dl_first_pkg)" ] ; then
      ALT_INST_LIST="debian/tmpl"
      [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
   fi
   if [ -s "$INST_LIST" ] ; then
      mkdir -p "debian/$p/${LLXCFG_JUMBLE_TEMPLATES}/"
      cat "$INST_LIST" |while read TDIR ; do
         if [ "$TDIR" ] ; then
            [ ! -d "$TDIR" ] && dl_exit_error "jumble templates directory \"${TDIR}\" not found"
            rsync -ax "$TDIR/" "debian/$p/${LLXCFG_JUMBLE_TEMPLATES}/"
            dl_add_depends_runtime "$p"
            dl_add_update "$p" "jumble"
         fi
      done
   fi
   return 0
}

dl_install_extensions(){
   p="$1"
   for t in ${LLXEXT_TYPES} ; do
      INST_LIST="debian/${p}.${t}-extensions"
      if [ "$p" = "$(dl_first_pkg)" ] ; then
         ALT_INST_LIST="debian/${t}-extensions"
         [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
      fi
      if [ -s "$INST_LIST" ] ; then
         cat "$INST_LIST" |while read line ; do
            dl_read_pkgline $line
            SCR_FILE="${DL_PKG_LINE[0]}"
            SCR_NAME="$(basename ${SCR_FILE})"
            SCR_NUM="${DL_PKG_LINE[1]}"
            [ ! -e "$SCR_FILE" ] && dl_exit_error "${t} script \"${SCR_FILE}\" not found"
            mkdir -p "debian/$p/$(llxscr_dir "$t")"
            mkdir -p "debian/$p/$(llxscr_endir "$t")"
            cp "$SCR_FILE" "debian/$p/$(llxscr_file "$t" "$SCR_NAME")"
            chmod +x "debian/$p/$(llxscr_file "$t" "$SCR_NAME")"
            ln -s "$(llxscr_relfile "${t}" "$SCR_NAME")" "debian/$p/$(llxscr_enfile "${t}" "$SCR_NAME" $SCR_NUM)"
            dl_add_depends_runtime "$p"
         done
      fi
   done
}


dl_cdd_test(){
   if echo "$1" |grep -q "^$CDD_FULL_PREFIX" ; then
      return 0
   fi
   return 1
}

dl_get_taskname(){
   local TASK_PART TASK_NAME
   if dl_cdd_test "$1" ; then
      # lliurex-cdd... packages have special task name convention
      TASK_PART="${1#$CDD_FULL_PREFIX}"
      TASK_NAME=""
      if [ "$TASK_PART" ] ; then
         TASK_NAME="${TASK_PART#-}"
      fi
      echo "$TASK_NAME"
   else
      echo "$1"
   fi
}

dl_get_cddpath(){
   local TASK_NAME CDD_PACKAGE_PATH
   CDD_PACKAGE_PATH="debian/${1}/${CDD_PATH}"
   TASK_NAME="$(dl_get_taskname "$1")"
   [ -z "${TASK_NAME}" ] || CDD_PACKAGE_PATH="${CDD_PACKAGE_PATH}/tasks/${TASK_NAME}"
   echo "${CDD_PACKAGE_PATH}"
}

dl_install_cddversion(){
   local CDD_VERSION CDD_PACKAGE_PATH NUM_VERSION
   CDD_VERSION="$(dl_show_field X-CDDVersion "$1")" || true
   CDD_PACKAGE_PATH="$(dl_get_cddpath "$1")"
   if [ "$CDD_VERSION" ] ; then
      mkdir -p "$CDD_PACKAGE_PATH"
      echo "$CDD_VERSION" |tr "," "\n" |sed -e "s%^[[:blank:]]*%%;s%[[:blank:]]*$%%" |sort -u >  "${CDD_PACKAGE_PATH}/cddversion"
   fi
   if [ "$CDD_VERSION" ] || dl_cdd_test "$1" ; then
      NUM_VERSION="$(dl_get_version)"
      mkdir -p "$CDD_PACKAGE_PATH"
      echo "${NUM_VERSION}" > "${CDD_PACKAGE_PATH}/version"
   fi

}

dl_install_pe(){
   # copy files in <binary_package_name> under 'PE_PATH' (/usr/share/pe/) and add pe --update to mantainer scripts
   for PETYPE in $PE_EXT_USERS $PE_EXT_GROUPS; do
      p="$1"
      for PREFIX in "" "_" ; do
      INST_LIST="debian/${p}.${PREFIX}pe-${PETYPE}"
      if [ "$p" = "$(dl_first_pkg)" ] ; then
         ALT_INST_LIST="debian/${PREFIX}pe-${PETYPE}"
         [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
      fi
      if [ -s "$INST_LIST" ] ; then
         cat "$INST_LIST" |while read line ; do
            dl_read_pkgline $line
            PE_FULLCMD="${DL_PKG_LINE[0]}"
            if [ "$PE_FULLCMD" ] ; then
               PE_BASENAME="$(basename "$PE_FULLCMD")"
               PE_DATA="$(echo "$line" |sed -e "s%^[^[:blank:]]\+[[:blank:]]*%%;s%[[:blank:]]*$%%")"
               PE_BASENAME="$(basename "$PE_FULLCMD")"
               PE_DIR="debian/$p/${PE_CFG_PATH}/$PE_BASENAME"
               mkdir -p "$PE_DIR"
               echo "$PE_FULLCMD" >"$PE_DIR/$p.$PE_EXT_CMD"
               echo "$PE_DATA" |tr "," "\n" |sort -u >> "$PE_DIR/$p.$PETYPE"
               TMP_DATA="$(tempfile)"
               sort -u "$PE_DIR/$p.$PETYPE" > "$TMP_DATA"
               cat "$TMP_DATA" > "$PE_DIR/$p.$PETYPE"
               rm "$TMP_DATA"
            fi
         done
      fi
      done
      rm -f "debian/${p}._pe-${PETYPE}" "debian/_pe-${PETYPE}"
   done
   if [ -d "debian/$p/${PE_CFG_PATH}" ] ; then
      dl_add_postinst "dllxh_installpe" "$p" "$PE_APP_UPDATE"
      dl_add_postrm "dllxh_installpe" "$p" "$PE_APP_UPDATE"
      dl_add_depends_runtime "$p"
   fi
   return 0
}

dl_pe_group(){
# package command group list
   f="debian/${p}._pe-${PE_EXT_GROUPS}"
   shift
   PROG="$1"
   shift
   echo "$PROG" "$@" >> "$f"
}

dl_pe_user(){
# package command user list
   f="debian/${p}._pe-${PE_EXT_USERS}"
   shift
   PROG="$1"
   shift
   echo "$PROG" "$@" >> "$f"
}

dl_subst_vars(){
   p="$1"
   touch debian/$p.substvars
   CURRENT_TYPES=$(find debian -maxdepth 1 -xtype f -name "*subst-?*" |sed -e "s%^debian/%%;s%^.*subst\-%%;s%^_*%%;s%_.*$%%"|tr "[:upper:]" "[:lower:]" |sort -u)
   for t in ${CURRENT_TYPES} ; do
      tt="${t#?}"
      TT="$(echo ${t%$tt}|tr "[:lower:]" "[:upper:]")"
      T="$TT$tt"
      for s in _${t} _${t}_${DEB_BUILD_ARCH} ${t} ${t}_${DEB_BUILD_ARCH} ${T} ${T}_${DEB_BUILD_ARCH} ; do
         INST_LIST="debian/${p}.subst-${s}"
         if [ "$p" = "$(dl_first_pkg)" ] ; then
            ALT_INST_LIST="debian/subst-${s}"
            [ -s "$ALT_INST_LIST" ] && INST_LIST="$ALT_INST_LIST"
         fi
         [ ! -r "$INST_LIST" ] || cat "$INST_LIST" |tr "," "\n" |sed -e "s%^[[:blank:]]*%%;s%[[:blank:]]*$%%" >> "debian/${p}.subst-__${t}_${DEB_BUILD_ARCH}"
      done
      [ ! -r "debian/${p}.subst-__${t}_${DEB_BUILD_ARCH}" ] ||echo "llx:$T=$(cat "debian/${p}.subst-__${t}_${DEB_BUILD_ARCH}" |sed "/^[[:blank:]]*$/d;s%^% %" |sort -u |tr "\n" ",")" >> debian/${p}.substvars
      rm -f "debian/${p}.subst-__${t}_${DEB_BUILD_ARCH}" "debian/${p}.subst-_${t}_${DEB_BUILD_ARCH}" "debian/${p}._${t}"
   done
}

dl_add_var(){
   VARNAME="$1"
   shift
   PKG_NAME="$1"
   shift
   VARVALUE="$1"
   echo "$VARVALUE" >> "debian/${PKG_NAME}.subst-_${VARNAME}_${DEB_BUILD_ARCH}"
}

dl_add_depends(){
   dl_add_var "depends" "$@"
}

dl_add_depends_runtime(){
   dl_add_depends "$1" "llxcfg-runtime"
}
